@@ -1,12 +1,12 @@
 /*
   Copyright @ 2018 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
-  
+
   You may not use this file except in compliance with the License.  You may
   obtain a copy of the License at
-  
+
     https://imagemagick.org/script/license.php
-  
+
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -22,7 +22,17 @@
 #define FUZZ_MAX_SIZE 2048
 #endif
 
-class FuzzingInitializer {
+static bool IsInvalidSize(const size_t size, const size_t min = 1, const size_t max = MagickPathExtent)
+{
+  if (size < min)
+    return true;
+  if (max > 0 && size > max)
+    return true;
+  return false;
+}
+
+class FuzzingInitializer
+{
 public:
   FuzzingInitializer() {
 
