@@ -74,7 +74,7 @@
 #include "MagickCore/module.h"
 #include "MagickCore/utility.h"
 #if defined(MAGICKCORE_HEIC_DELEGATE)
-#if defined(MAGICKCORE_WINDOWS_SUPPORT)
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) && !defined(__MINGW32__)
 #include <heif.h>
 #else
 #include <libheif/heif.h>
@@ -86,19 +86,19 @@
   Define declarations.
 */
 #define XmpNamespaceExtent  28
-
+
 /*
   Const declarations.
 */
 static const char
   xmp_namespace[] = "http://ns.adobe.com/xap/1.0/ ";
-
+
 /*
   Forward declarations.
 */
 static MagickBooleanType
   WriteHEICImage(const ImageInfo *,Image *,ExceptionInfo *);
-
+
 /*x
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -140,7 +140,6 @@ static inline MagickBooleanType IsHEIFSuccess(Image *image,
 static MagickBooleanType ReadHEICColorProfile(Image *image,
   struct heif_image_handle *image_handle,ExceptionInfo *exception)
 {
-#if LIBHEIF_NUMERIC_VERSION >= 0x01040000
   size_t
     length;
 
@@ -176,7 +175,6 @@ static MagickBooleanType ReadHEICColorProfile(Image *image,
         }
     }
   color_profile=(unsigned char *) RelinquishMagickMemory(color_profile);
-#endif
   return(MagickTrue);
 }
 
@@ -190,7 +188,7 @@ static MagickBooleanType ReadHEICExifProfile(Image *image,
     count;
 
   size_t
-    extent;
+    length;
 
   struct heif_error
     error;
@@ -205,11 +203,13 @@ static MagickBooleanType ReadHEICExifProfile(Image *image,
     &id,1);
   if (count != 1)
     return(MagickTrue);
-  extent=heif_image_handle_get_metadata_size(image_handle,id);
-  if ((MagickSizeType) extent > GetBlobSize(image))
+  length=heif_image_handle_get_metadata_size(image_handle,id);
+  if (length <= 8)
+    return(MagickTrue);
+  if ((MagickSizeType) length > GetBlobSize(image))
     ThrowBinaryException(CorruptImageError,"InsufficientImageDataInFile",
       image->filename);
-  exif_profile=(unsigned char *) AcquireQuantumMemory(1,extent);
+  exif_profile=(unsigned char *) AcquireQuantumMemory(1,length);
   if (exif_profile == (unsigned char *) NULL)
     return(MagickFalse);
   error=heif_image_handle_get_metadata(image_handle,id,exif_profile);
@@ -221,9 +221,7 @@ static MagickBooleanType ReadHEICExifProfile(Image *image,
       /*
         Skip first 4 bytes, the offset to the TIFF header.
       */
-      profile=(StringInfo*) NULL;
-      if (extent > 8)
-        profile=BlobToStringInfo(exif_profile+4,(size_t) extent-4);
+      profile=BlobToStringInfo(exif_profile+4,(size_t) length-4);
       if (profile != (StringInfo*) NULL)
         {
           (void) SetImageProfile(image,"exif",profile,exception);
@@ -262,6 +260,7 @@ static MagickBooleanType ReadHEICImageHandle(const ImageInfo *image_info,
     chroma;
 
   int
+    bits_per_pixel,
     shift,
     stride = 0;
 
@@ -286,19 +285,12 @@ static MagickBooleanType ReadHEICImageHandle(const ImageInfo *image_info,
   */
   image->columns=(size_t) heif_image_handle_get_width(image_handle);
   image->rows=(size_t) heif_image_handle_get_height(image_handle);
-  image->depth=8;
-#if LIBHEIF_NUMERIC_VERSION > 0x01040000
-  {
-    int
-      bits_per_pixel;
-
-    bits_per_pixel=heif_image_handle_get_luma_bits_per_pixel(image_handle);
-    if (bits_per_pixel != -1)
-      image->depth=(size_t) bits_per_pixel;
-  }
-#endif
-  if (heif_image_handle_has_alpha_channel(image_handle))
+  if (heif_image_handle_has_alpha_channel(image_handle) != 0)
     image->alpha_trait=BlendPixelTrait;
+  image->depth=8;
+  bits_per_pixel=heif_image_handle_get_luma_bits_per_pixel(image_handle);
+  if (bits_per_pixel != -1)
+    image->depth=(size_t) bits_per_pixel;
   preserve_orientation=IsStringTrue(GetImageOption(image_info,
     "heic:preserve-orientation"));
   if (preserve_orientation == MagickFalse)
@@ -328,8 +320,6 @@ static MagickBooleanType ReadHEICImageHandle(const ImageInfo *image_info,
     }
   error=heif_decode_image(image_handle,&heif_image,heif_colorspace_RGB,chroma,
     decode_options);
-  if (IsHEIFSuccess(image,&error,exception) == MagickFalse)
-    return(MagickFalse);
   heif_decoding_options_free(decode_options);
   if (IsHEIFSuccess(image,&error,exception) == MagickFalse)
     return(MagickFalse);
@@ -461,6 +451,9 @@ static void ReadHEICDepthImage(const ImageInfo *image_info,Image *image,
 static Image *ReadHEICImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
+  enum heif_filetype_result
+    filetype_check;
+
   heif_item_id
     primary_image_id;
 
@@ -470,9 +463,8 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  size_t
-    count,
-    length;
+  ssize_t
+    count;
 
   struct heif_context
     *heif_context;
@@ -483,55 +475,50 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
   struct heif_image_handle
     *image_handle;
 
-  void
-    *container;
+  unsigned char
+    magic[12];
 
   /*
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickCoreSignature);
-  if (image_info->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
-      image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
   image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(DestroyImageList(image));
-  if (GetBlobSize(image) > (MagickSizeType) MAGICK_SSIZE_MAX)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-  length=(size_t) GetBlobSize(image);
-  container=AcquireMagickMemory(length);
-  if (container == (void *) NULL)
-    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-  if (ReadBlob(image,length,container) != (ssize_t) length)
-    {
-      container=RelinquishMagickMemory(container);
-      ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
-    }
+  if (ReadBlob(image,sizeof(magic),magic) != sizeof(magic))
+    ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
+  filetype_check=heif_check_filetype(magic,sizeof(magic));
+  if (filetype_check == heif_filetype_no)
+    ThrowReaderException(CoderError,"ImageTypeNotSupported");
+  (void) CloseBlob(image);
 #if LIBHEIF_NUMERIC_VERSION >= 0x010b0000
-  if (heif_has_compatible_brand((unsigned char *) container,(int) length,"avif") != MagickFalse)
+  if (heif_has_compatible_brand(magic,sizeof(magic), "avif") == 1)
     (void) CopyMagickString(image->magick,"AVIF",MagickPathExtent);
 #endif
   /*
     Decode HEIF image.
   */
   heif_context=heif_context_alloc();
-  error=heif_context_read_from_memory_without_copy(heif_context,container,
-    length,NULL);
+  if (heif_context == (struct heif_context *) NULL)
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  error=heif_context_read_from_file(heif_context,image->filename,
+    (const struct heif_reading_options *) NULL);
   if (IsHEIFSuccess(image,&error,exception) == MagickFalse)
     {
       heif_context_free(heif_context);
-      container=RelinquishMagickMemory(container);
       return(DestroyImageList(image));
     }
   error=heif_context_get_primary_image_ID(heif_context,&primary_image_id);
   if (IsHEIFSuccess(image,&error,exception) == MagickFalse)
     {
       heif_context_free(heif_context);
-      container=RelinquishMagickMemory(container);
       return(DestroyImageList(image));
     }
   error=heif_context_get_image_handle(heif_context,primary_image_id,
@@ -539,7 +526,6 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
   if (IsHEIFSuccess(image,&error,exception) == MagickFalse)
     {
       heif_context_free(heif_context);
-      container=RelinquishMagickMemory(container);
       return(DestroyImageList(image));
     }
   status=ReadHEICImageHandle(image_info,image,image_handle,exception);
@@ -550,14 +536,13 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
       heif_item_id
         *ids;
 
-      size_t
+      ssize_t
         i;
 
       ids=(heif_item_id *) AcquireQuantumMemory((size_t) count,sizeof(*ids));
       if (ids == (heif_item_id *) NULL)
         {
           heif_context_free(heif_context);
-          container=RelinquishMagickMemory(container);
           return(DestroyImageList(image));
         }
       (void) heif_context_get_list_of_top_level_image_IDs(heif_context,ids,
@@ -597,16 +582,13 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
   if (IsHEIFSuccess(image,&error,exception) == MagickFalse)
     {
       heif_context_free(heif_context);
-      container=RelinquishMagickMemory(container);
       return(DestroyImageList(image));
     }
   ReadHEICDepthImage(image_info,image,image_handle,exception);
   heif_image_handle_release(image_handle);
   heif_context_free(heif_context);
-  container=RelinquishMagickMemory(container);
   if (status == MagickFalse)
     return(DestroyImageList(image));
-  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 #endif
@@ -639,7 +621,6 @@ static Image *ReadHEICImage(const ImageInfo *image_info,
 static MagickBooleanType IsHEIC(const unsigned char *magick,const size_t length)
 {
 #if defined(MAGICKCORE_HEIC_DELEGATE)
-#if LIBHEIF_NUMERIC_VERSION >= 0x01040000
   enum heif_filetype_result
     type;
 
@@ -648,7 +629,6 @@ static MagickBooleanType IsHEIC(const unsigned char *magick,const size_t length)
   type=heif_check_filetype(magick,(int) length);
   if (type == heif_filetype_yes_supported)
     return(MagickTrue);
-#endif
 #endif
   return(MagickFalse);
 }
@@ -683,12 +663,8 @@ ModuleExport size_t RegisterHEICImage(void)
   entry=AcquireMagickInfo("HEIC","HEIC","High Efficiency Image Format");
 #if defined(MAGICKCORE_HEIC_DELEGATE)
   entry->decoder=(DecodeImageHandler *) ReadHEICImage;
-#if LIBHEIF_NUMERIC_VERSION >= 0x01030000
   if (heif_have_encoder_for_format(heif_compression_HEVC))
     entry->encoder=(EncodeImageHandler *) WriteHEICImage;
-#else
-  entry->encoder=(EncodeImageHandler *) WriteHEICImage;
-#endif
 #endif
   entry->magick=(IsImageFormatHandler *) IsHEIC;
   entry->mime_type=ConstantString("image/heic");
@@ -696,16 +672,13 @@ ModuleExport size_t RegisterHEICImage(void)
   entry->version=ConstantString(LIBHEIF_VERSION);
 #endif
   entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   (void) RegisterMagickInfo(entry);
   entry=AcquireMagickInfo("HEIC","HEIF","High Efficiency Image Format");
 #if defined(MAGICKCORE_HEIC_DELEGATE)
   entry->decoder=(DecodeImageHandler *) ReadHEICImage;
-#if LIBHEIF_NUMERIC_VERSION >= 0x01030000
   if (heif_have_encoder_for_format(heif_compression_HEVC))
     entry->encoder=(EncodeImageHandler *) WriteHEICImage;
-#else
-  entry->encoder=(EncodeImageHandler *) WriteHEICImage;
-#endif
 #endif
   entry->magick=(IsImageFormatHandler *) IsHEIC;
   entry->mime_type=ConstantString("image/heif");
@@ -713,6 +686,7 @@ ModuleExport size_t RegisterHEICImage(void)
   entry->version=ConstantString(LIBHEIF_VERSION);
 #endif
   entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   (void) RegisterMagickInfo(entry);
 #if LIBHEIF_NUMERIC_VERSION > 0x01060200
   entry=AcquireMagickInfo("HEIC","AVIF","AV1 Image File Format");
@@ -728,6 +702,7 @@ ModuleExport size_t RegisterHEICImage(void)
   entry->version=ConstantString(LIBHEIF_VERSION);
 #endif
   entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderBlobSupportFlag;
   (void) RegisterMagickInfo(entry);
 #endif
   return(MagickImageCoderSignature);
@@ -790,7 +765,6 @@ ModuleExport void UnregisterHEICImage(void)
 */
 
 #if defined(MAGICKCORE_HEIC_DELEGATE)
-#if LIBHEIF_NUMERIC_VERSION >= 0x01030000
 static void WriteProfile(struct heif_context *context,Image *image,
   ExceptionInfo *exception)
 {
@@ -869,7 +843,6 @@ static void WriteProfile(struct heif_context *context,Image *image,
   }
   heif_image_handle_release(image_handle);
 }
-#endif
 
 static struct heif_error heif_write_func(struct heif_context *context,
   const void* data,size_t size,void* userdata)
@@ -1167,9 +1140,6 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
   struct heif_image
     *heif_image = (struct heif_image*) NULL;
 
-  struct heif_writer
-    writer;
-
   /*
     Open output image file.
   */
@@ -1177,7 +1147,7 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
   assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickCoreSignature);
-  if (image->debug != MagickFalse)
+  if (IsEventLogging() != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -1190,10 +1160,8 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
 #endif
   do
   {
-#if LIBHEIF_NUMERIC_VERSION >= 0x01040000
     const StringInfo
       *profile;
-#endif
 
     enum heif_chroma
       chroma;
@@ -1258,12 +1226,10 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
     status=IsHEIFSuccess(image,&error,exception);
     if (status == MagickFalse)
       break;
-#if LIBHEIF_NUMERIC_VERSION >= 0x01040000
     profile=GetImageProfile(image,"icc");
     if (profile != (StringInfo *) NULL)
       (void) heif_image_set_raw_color_profile(heif_image,"prof",
         GetStringInfoDatum(profile),GetStringInfoLength(profile));
-#endif
     if (colorspace == heif_colorspace_YCbCr)
       status=WriteHEICImageYCbCr(image,heif_image,exception);
     else
@@ -1284,8 +1250,6 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
       else
         error=heif_encoder_set_lossy_quality(heif_encoder,(int)
           image_info->quality);
-    if (IsHEIFSuccess(image,&error,exception) == MagickFalse)
-      break;
     status=IsHEIFSuccess(image,&error,exception);
     if (status == MagickFalse)
       break;
@@ -1321,10 +1285,8 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
     status=IsHEIFSuccess(image,&error,exception);
     if (status == MagickFalse)
       break;
-#if LIBHEIF_NUMERIC_VERSION >= 0x01030000
     if (image->profiles != (void *) NULL)
       WriteProfile(heif_context,image,exception);
-#endif
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
@@ -1340,12 +1302,11 @@ static MagickBooleanType WriteHEICImage(const ImageInfo *image_info,
   } while (image_info->adjoin != MagickFalse);
   if (status != MagickFalse)
     {
+      struct heif_writer
+        writer;
+
       writer.writer_api_version=1;
       writer.write=heif_write_func;
-#if LIBHEIF_NUMERIC_VERSION >= 0x01030000
-      if (image->profiles != (void *) NULL)
-        WriteProfile(heif_context,image,exception);
-#endif
       error=heif_context_write(heif_context,&writer,image);
       status=IsHEIFSuccess(image,&error,exception);
     }
